/*
 * BigPlotWindow.java
 *
 * Created on March 27, 2007, 10:17 AM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * This window display the cartesian RF-Phi plot.
 * @author cwarren
 */
public class BigPlotWindow extends javax.swing.JFrame
{
    private PebblePlotterCartesian plot;
    private FileFilterImage filterImage = new FileFilterImage();

    // for saves and loads
    private final JFileChooser fileChooser = new JFileChooser ();
    
    /** Creates new form BigPlotWindow */
    public BigPlotWindow ()
    {
        initComponents ();

        //--------------------------
        //imageFormatWin = new ImageFormatWindow();
        //imageFormatWin.setVisible (false);
        String displayText = "    X:       Y:                    ";
        this.MouseInfoLabel.setText (displayText.substring (0,23));
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        XMaxRadioGroup = new javax.swing.ButtonGroup();
        CloseWindowButton = new javax.swing.JButton();
        Space2Label2 = new javax.swing.JLabel();
        MainDisplayPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        LogScaleToggleButton = new javax.swing.JToggleButton();
        Space2Label1 = new javax.swing.JLabel();
        XMaxLabel = new javax.swing.JLabel();
        XMaxRadio1Auto = new javax.swing.JRadioButton();
        XMaxRadio2Manual = new javax.swing.JRadioButton();
        XMaxTextField = new javax.swing.JTextField();
        Space2Label = new javax.swing.JLabel();
        PebbleInfoToggleButton = new javax.swing.JToggleButton();
        Space2Label3 = new javax.swing.JLabel();
        SnapshotButton = new javax.swing.JButton();
        MouseInfoLabel = new javax.swing.JLabel();

        CloseWindowButton.setText("Close");
        CloseWindowButton.setAlignmentX(1.0F);
        CloseWindowButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                CloseWindowButtonActionPerformed(evt);
            }
        });

        Space2Label2.setText("          ");

        getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.Y_AXIS));

        setTitle("GeoShear - RF vs Phi - Big Plot");
        setAlwaysOnTop(true);
        addComponentListener(new java.awt.event.ComponentAdapter()
        {
            public void componentResized(java.awt.event.ComponentEvent evt)
            {
                formComponentResized(evt);
            }
        });

        MainDisplayPanel.setLayout(new javax.swing.BoxLayout(MainDisplayPanel, javax.swing.BoxLayout.X_AXIS));

        MainDisplayPanel.setBackground(new java.awt.Color(255, 255, 255));
        MainDisplayPanel.setMaximumSize(new java.awt.Dimension(20000, 10000));
        MainDisplayPanel.setMinimumSize(new java.awt.Dimension(50, 50));
        MainDisplayPanel.addComponentListener(new java.awt.event.ComponentAdapter()
        {
            public void componentResized(java.awt.event.ComponentEvent evt)
            {
                MainDisplayPanelComponentResized(evt);
            }
        });
        MainDisplayPanel.addMouseListener(new java.awt.event.MouseAdapter()
        {
            public void mouseClicked(java.awt.event.MouseEvent evt)
            {
                MainDisplayPanelMouseClicked(evt);
            }
        });

        getContentPane().add(MainDisplayPanel);

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.X_AXIS));

        LogScaleToggleButton.setText("log scale");
        LogScaleToggleButton.setToolTipText("Use a log10 scale for X values");
        LogScaleToggleButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                LogScaleToggleButtonActionPerformed(evt);
            }
        });

        jPanel1.add(LogScaleToggleButton);

        Space2Label1.setText("   ");
        jPanel1.add(Space2Label1);

        XMaxLabel.setText("X Max:");
        jPanel1.add(XMaxLabel);

        XMaxRadioGroup.add(XMaxRadio1Auto);
        XMaxRadio1Auto.setSelected(true);
        XMaxRadio1Auto.setText("Auto ");
        XMaxRadio1Auto.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        XMaxRadio1Auto.setMargin(new java.awt.Insets(0, 0, 0, 0));
        XMaxRadio1Auto.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                XMaxRadio1AutoActionPerformed(evt);
            }
        });
        XMaxRadio1Auto.addChangeListener(new javax.swing.event.ChangeListener()
        {
            public void stateChanged(javax.swing.event.ChangeEvent evt)
            {
                XMaxRadio1AutoStateChanged(evt);
            }
        });

        jPanel1.add(XMaxRadio1Auto);

        XMaxRadioGroup.add(XMaxRadio2Manual);
        XMaxRadio2Manual.setText("Manual");
        XMaxRadio2Manual.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        XMaxRadio2Manual.setMargin(new java.awt.Insets(0, 0, 0, 0));
        XMaxRadio2Manual.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                XMaxRadio2ManualActionPerformed(evt);
            }
        });
        XMaxRadio2Manual.addChangeListener(new javax.swing.event.ChangeListener()
        {
            public void stateChanged(javax.swing.event.ChangeEvent evt)
            {
                XMaxRadio2ManualStateChanged(evt);
            }
        });

        jPanel1.add(XMaxRadio2Manual);

        XMaxTextField.setText("3.0");
        XMaxTextField.setMaximumSize(new java.awt.Dimension(40, 20));
        XMaxTextField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                XMaxTextFieldActionPerformed(evt);
            }
        });
        XMaxTextField.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusGained(java.awt.event.FocusEvent evt)
            {
                XMaxTextFieldFocusGained(evt);
            }
        });

        jPanel1.add(XMaxTextField);

        Space2Label.setText("      ");
        jPanel1.add(Space2Label);

        PebbleInfoToggleButton.setText("Pebble Info");
        PebbleInfoToggleButton.setToolTipText("Show info on selected pebbles");
        PebbleInfoToggleButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                PebbleInfoToggleButtonActionPerformed(evt);
            }
        });

        jPanel1.add(PebbleInfoToggleButton);

        Space2Label3.setText("   ");
        jPanel1.add(Space2Label3);

        SnapshotButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_camera_small_20x15.gif")));
        SnapshotButton.setToolTipText("Snapshot");
        SnapshotButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                SnapshotButtonActionPerformed(evt);
            }
        });

        jPanel1.add(SnapshotButton);

        MouseInfoLabel.setFont(new java.awt.Font("Courier New", 0, 11));
        MouseInfoLabel.setText("  RF:      Phi:    ");
        jPanel1.add(MouseInfoLabel);

        getContentPane().add(jPanel1);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Displays at the bottom of the window the X and Y location of the click, 
     * based on the plot's coordinate system.
     */
    private void MainDisplayPanelMouseClicked (java.awt.event.MouseEvent evt)//GEN-FIRST:event_MainDisplayPanelMouseClicked
    {//GEN-HEADEREND:event_MainDisplayPanelMouseClicked
        //JOptionPane.showMessageDialog (this,"mouse clicked at "+evt.getX () + "," + evt.getY ());
        double RF = evt.getX () - Math.min (this.getPlot ().getXAxis ().x1,this.getPlot ().getXAxis ().x2);
        double Phi = Math.max (this.getPlot ().getXAxis ().y1,this.getPlot ().getXAxis ().y2) - evt.getY ();
        
        if (RF > 0 && Phi > 0)
        {
            double xSize = this.getPlot ().getXAxis ().getP1 ().distance (this.getPlot ().getXAxis ().getP2 ());
            double ySize = this.getPlot ().getYAxis ().getP1 ().distance (this.getPlot ().getYAxis ().getP2 ());
            RF = RF/xSize;
            Phi = Phi/ySize;
            
            RF = RF * Double.parseDouble (this.getPlot ().getXMaxValLabel ());
            Phi = Phi * Math.PI - (Math.PI / 2.0);
            
            RF = Utils.truncForDisplay (RF);
            Phi = Utils.truncForDisplay (Phi);
            String displayText = "    X:"+RF+"  Y:"+Phi+"             ";
            this.MouseInfoLabel.setText (displayText.substring (0,23));
        }
    }//GEN-LAST:event_MainDisplayPanelMouseClicked

    private void PebbleInfoToggleButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_PebbleInfoToggleButtonActionPerformed
    {//GEN-HEADEREND:event_PebbleInfoToggleButtonActionPerformed
        this.plot.setShowRFOnPebble (this.PebbleInfoToggleButton.isSelected ());
        this.plot.repaint ();
    }//GEN-LAST:event_PebbleInfoToggleButtonActionPerformed

    private void LogScaleToggleButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_LogScaleToggleButtonActionPerformed
    {//GEN-HEADEREND:event_LogScaleToggleButtonActionPerformed
        ((PlotRfVsPhi)(this.plot)).setLogScaleX (LogScaleToggleButton.isSelected ());
        this.repaint ();

    }//GEN-LAST:event_LogScaleToggleButtonActionPerformed

    /**
     * Captures the screen region of this window as an image and provides a dialog for 
     * the user to save that image.
     */
    private void SnapshotButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_SnapshotButtonActionPerformed
    {//GEN-HEADEREND:event_SnapshotButtonActionPerformed
        try
        {                                              
            Toolkit toolkit = Toolkit.getDefaultToolkit();
            Robot robot = new Robot();
            BufferedImage image = robot.createScreenCapture (new Rectangle(this.plot.getLocationOnScreen (), this.plot.getSize ()));

            //this.imageFormatWin.setVisible (true);
            
            fileChooser.setFileFilter(filterImage);

            int returnVal = fileChooser.showSaveDialog (this);
            if (returnVal == JFileChooser.APPROVE_OPTION)
            {
                File saveFile = fileChooser.getSelectedFile ();
                if (filterImage.accept (saveFile))
                {
                    ImageIO.write (image,Utils.getExtension (saveFile),saveFile);
                    JOptionPane.showMessageDialog (this,"Saved to "+saveFile.getCanonicalPath ());
                } else
                {
                    toolkit.beep ();
                    JOptionPane.showMessageDialog (this,"Unsupported format, only "+filterImage.getDescription ()+". Save aborted.");
                }
            }
        }
        catch (Exception exc)
        {
            exc.printStackTrace ();
        }
        //this.imageFormatWin.setVisible (false);
        this.repaint ();
    }//GEN-LAST:event_SnapshotButtonActionPerformed
        
    private void XMaxRadio2ManualActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_XMaxRadio2ManualActionPerformed
    {//GEN-HEADEREND:event_XMaxRadio2ManualActionPerformed
        this.plot.setManualXMax (this.XMaxRadio2Manual.isSelected ());
        this.plot.repaint ();
    }//GEN-LAST:event_XMaxRadio2ManualActionPerformed
    
    private void XMaxRadio1AutoActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_XMaxRadio1AutoActionPerformed
    {//GEN-HEADEREND:event_XMaxRadio1AutoActionPerformed
        this.plot.setManualXMax (this.XMaxRadio2Manual.isSelected ());
        this.plot.repaint ();
    }//GEN-LAST:event_XMaxRadio1AutoActionPerformed
    
    private void XMaxRadio2ManualStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_XMaxRadio2ManualStateChanged
    {//GEN-HEADEREND:event_XMaxRadio2ManualStateChanged
        this.plot.setManualXMax (this.XMaxRadio2Manual.isSelected ());
        this.plot.repaint ();
    }//GEN-LAST:event_XMaxRadio2ManualStateChanged
    
    private void XMaxTextFieldActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_XMaxTextFieldActionPerformed
    {//GEN-HEADEREND:event_XMaxTextFieldActionPerformed
        try
        {
            this.plot.setXMaxValManual (Double.parseDouble (this.XMaxTextField.getText ()));
            this.plot.setXMaxValLabel (this.XMaxTextField.getText ());
        }
        catch (NumberFormatException exc)
        { // invalid entries are just ignored
        }
        this.plot.setManualXMax (this.XMaxRadio2Manual.isSelected ());
        this.plot.repaint ();
    }//GEN-LAST:event_XMaxTextFieldActionPerformed
    
    private void XMaxTextFieldFocusGained (java.awt.event.FocusEvent evt)//GEN-FIRST:event_XMaxTextFieldFocusGained
    {//GEN-HEADEREND:event_XMaxTextFieldFocusGained
        this.XMaxRadio2Manual.setSelected (true);
        this.plot.setManualXMax (this.XMaxRadio2Manual.isSelected ());
        this.plot.repaint ();
    }//GEN-LAST:event_XMaxTextFieldFocusGained
    
    private void XMaxRadio1AutoStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_XMaxRadio1AutoStateChanged
    {//GEN-HEADEREND:event_XMaxRadio1AutoStateChanged
        this.plot.setManualXMax (this.XMaxRadio2Manual.isSelected ());
        this.plot.repaint ();
    }//GEN-LAST:event_XMaxRadio1AutoStateChanged
        
    private void MainDisplayPanelComponentResized (java.awt.event.ComponentEvent evt)//GEN-FIRST:event_MainDisplayPanelComponentResized
    {//GEN-HEADEREND:event_MainDisplayPanelComponentResized
        this.fixPlotSizing ();
    }//GEN-LAST:event_MainDisplayPanelComponentResized
    
    private void formComponentResized (java.awt.event.ComponentEvent evt)//GEN-FIRST:event_formComponentResized
    {//GEN-HEADEREND:event_formComponentResized
        this.fixPlotSizing ();
    }//GEN-LAST:event_formComponentResized
    
    private void CloseWindowButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_CloseWindowButtonActionPerformed
    {//GEN-HEADEREND:event_CloseWindowButtonActionPerformed
        this.setVisible (false);
    }//GEN-LAST:event_CloseWindowButtonActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[])
    {
        java.awt.EventQueue.invokeLater (new Runnable ()
        {
            public void run ()
            {
                new BigPlotWindow ().setVisible (true);
            }
        });
    }
    
    /**
     * Returns the PebblePlotterCartesian object which this window displays
     * @return The PebblePlotterCartesian object which this window displays
     */
    public PebblePlotterCartesian getPlot ()
    {
        return plot;
    }
    
    /**
     * Sets the PebblePlotterCartesian which will display in this window
     * @param plot The PebblePlotterCartesian which will be displayed in this window
     */
    public void setPlot (PebblePlotterCartesian plot)
    {
        this.plot = plot;
        this.fixPlotSizing ();
        
        this.plot.setMarkFill (true);
        this.plot.setMarkSize (10);
        this.plot.setMarkShape (PebblePlotterCartesian.MARK_CIRCLE);
        
        
        this.MainDisplayPanel.add (plot);
        
        int x=1;
        
        
    }
    
    private void fixPlotSizing ()
    {
        if (this.plot != null)
        {
            this.plot.setMaximumSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setMinimumSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setPreferredSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setUpPlot ();
        }
    }
    
    
    /**
     * Gets the panel in which the PebblePlotterCartesian is displayed
     * @return The panel in which the PebblePlotterCartesian is displayed
     */
    public javax.swing.JPanel getMainDisplayPanel ()
    {
        return MainDisplayPanel;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton CloseWindowButton;
    private javax.swing.JToggleButton LogScaleToggleButton;
    private javax.swing.JPanel MainDisplayPanel;
    private javax.swing.JLabel MouseInfoLabel;
    private javax.swing.JToggleButton PebbleInfoToggleButton;
    private javax.swing.JButton SnapshotButton;
    private javax.swing.JLabel Space2Label;
    private javax.swing.JLabel Space2Label1;
    private javax.swing.JLabel Space2Label2;
    private javax.swing.JLabel Space2Label3;
    private javax.swing.JLabel XMaxLabel;
    private javax.swing.JRadioButton XMaxRadio1Auto;
    private javax.swing.JRadioButton XMaxRadio2Manual;
    private javax.swing.ButtonGroup XMaxRadioGroup;
    private javax.swing.JTextField XMaxTextField;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables
    
}
