/*
 * BigPolarWindow.java
 *
 * Created on June 28, 2007, 3:39 PM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * This window displays a polar plot of 2*Phi to logn(RF).
 * @author cwarren
 */
public class BigPolarWindow extends javax.swing.JFrame
{

    private PebblePlotterPolar plot;

    private FileFilterImage filterImage = new FileFilterImage();

    // for saves and loads
    private final JFileChooser fileChooser = new JFileChooser ();

    /** Creates new form BigPolarWindow */
    public BigPolarWindow ()
    {
        initComponents ();

        String displayText = "    r:       a:                    ";
        this.MouseInfoLabel.setText (displayText.substring (0,23));

    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        CloseButton = new javax.swing.JButton();
        MainDisplayPanel = new javax.swing.JPanel();
        ControlsPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        SnapshotButton = new javax.swing.JButton();
        MouseInfoLabel = new javax.swing.JLabel();

        CloseButton.setText("Close");
        CloseButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                CloseButtonActionPerformed(evt);
            }
        });

        getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.Y_AXIS));

        setTitle("GeoShear - RF vs 2*Phi Polar Plot");
        setAlwaysOnTop(true);
        addComponentListener(new java.awt.event.ComponentAdapter()
        {
            public void componentResized(java.awt.event.ComponentEvent evt)
            {
                formComponentResized(evt);
            }
        });

        MainDisplayPanel.setBackground(new java.awt.Color(255, 255, 255));
        MainDisplayPanel.addComponentListener(new java.awt.event.ComponentAdapter()
        {
            public void componentResized(java.awt.event.ComponentEvent evt)
            {
                MainDisplayPanelComponentResized(evt);
            }
        });
        MainDisplayPanel.addMouseListener(new java.awt.event.MouseAdapter()
        {
            public void mouseClicked(java.awt.event.MouseEvent evt)
            {
                MainDisplayPanelMouseClicked(evt);
            }
        });

        org.jdesktop.layout.GroupLayout MainDisplayPanelLayout = new org.jdesktop.layout.GroupLayout(MainDisplayPanel);
        MainDisplayPanel.setLayout(MainDisplayPanelLayout);
        MainDisplayPanelLayout.setHorizontalGroup(
            MainDisplayPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 410, Short.MAX_VALUE)
        );
        MainDisplayPanelLayout.setVerticalGroup(
            MainDisplayPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 275, Short.MAX_VALUE)
        );
        getContentPane().add(MainDisplayPanel);

        ControlsPanel.setLayout(new javax.swing.BoxLayout(ControlsPanel, javax.swing.BoxLayout.X_AXIS));

        jLabel1.setText("    ");
        ControlsPanel.add(jLabel1);

        SnapshotButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/icon_camera_small_20x15.gif")));
        SnapshotButton.setToolTipText("Snapshot");
        SnapshotButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                SnapshotButtonActionPerformed(evt);
            }
        });

        ControlsPanel.add(SnapshotButton);

        MouseInfoLabel.setFont(new java.awt.Font("Courier New", 0, 11));
        MouseInfoLabel.setText("    r:      a:      ");
        ControlsPanel.add(MouseInfoLabel);

        getContentPane().add(ControlsPanel);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Causes the coordinates of the mouse click to be display in eh bottom section of 
     * this window. The coordinates are in the radius,angle framework of the plot.
     */
    private void MainDisplayPanelMouseClicked (java.awt.event.MouseEvent evt)//GEN-FIRST:event_MainDisplayPanelMouseClicked
    {//GEN-HEADEREND:event_MainDisplayPanelMouseClicked
        double r = this.getPlot ().getPlotOrigin ().distance ((Point2D)evt.getPoint ());
        double a = Math.atan2 (evt.getY () - this.getPlot ().getPlotOrigin ().getY (),
                               evt.getX () - this.getPlot ().getPlotOrigin ().getX ());

        r = (r/this.getPlot ().getRadiusMaxDim ()) * this.getPlot ().getRadiusMax ();
        a = Math.toDegrees (a) * -1;
        
        r = Utils.truncForDisplay (r);
        a = Utils.truncForDisplay (a);
        String displayText = "    r:"+r+"  a:"+a+"             ";
        this.MouseInfoLabel.setText (displayText.substring (0,23));        
    }//GEN-LAST:event_MainDisplayPanelMouseClicked

    private void formComponentResized (java.awt.event.ComponentEvent evt)//GEN-FIRST:event_formComponentResized
    {//GEN-HEADEREND:event_formComponentResized
        this.fixPlotSizing ();
    }//GEN-LAST:event_formComponentResized

    private void MainDisplayPanelComponentResized (java.awt.event.ComponentEvent evt)//GEN-FIRST:event_MainDisplayPanelComponentResized
    {//GEN-HEADEREND:event_MainDisplayPanelComponentResized
        this.fixPlotSizing ();
    }//GEN-LAST:event_MainDisplayPanelComponentResized

    /**
     * Captures the screen region of the plot in this window and allows the user to
     * save that image to disk.
     */
    private void SnapshotButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_SnapshotButtonActionPerformed
    {//GEN-HEADEREND:event_SnapshotButtonActionPerformed
        try
        {                                              
            Toolkit toolkit = Toolkit.getDefaultToolkit();
            Robot robot = new Robot();
            BufferedImage image = robot.createScreenCapture (new Rectangle(this.getPlot().getLocationOnScreen (), this.getPlot().getSize ()));

            //this.imageFormatWin.setVisible (true);
            
            fileChooser.setFileFilter(filterImage);

            int returnVal = fileChooser.showSaveDialog (this);
            if (returnVal == JFileChooser.APPROVE_OPTION)
            {
                File saveFile = fileChooser.getSelectedFile ();
                if (filterImage.accept (saveFile))
                {
                    ImageIO.write (image,Utils.getExtension (saveFile),saveFile);
                    JOptionPane.showMessageDialog (this,"Saved to "+saveFile.getCanonicalPath ());
                } else
                {
                    toolkit.beep ();
                    JOptionPane.showMessageDialog (this,"Unsupported format, only "+filterImage.getDescription ()+". Save aborted.");
                }
            }
        }
        catch (Exception exc)
        {
            exc.printStackTrace ();
        }
        //this.imageFormatWin.setVisible (false);
        this.repaint ();
    }//GEN-LAST:event_SnapshotButtonActionPerformed

    private void CloseButtonActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_CloseButtonActionPerformed
    {//GEN-HEADEREND:event_CloseButtonActionPerformed
        this.setVisible (false);
    }//GEN-LAST:event_CloseButtonActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[])
    {
        java.awt.EventQueue.invokeLater (new Runnable ()
        {
            public void run ()
            {
                new BigPolarWindow ().setVisible (true);
            }
        });
    }

    public PebblePlotterPolar getPlot ()
    {
        return plot;
    }

    public void setPlot (PebblePlotterPolar plot)
    {
        this.plot = plot;
        this.fixPlotSizing ();
        
        this.plot.setMarkFill (true);
        this.plot.setMarkSize (10);
        this.plot.setMarkShape (PebblePlotterCartesian.MARK_CIRCLE);
        
        
        this.MainDisplayPanel.add (plot);
    }
    
    private void fixPlotSizing ()
    {
        if (this.plot != null)
        {
            this.plot.setMaximumSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setMinimumSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setPreferredSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setSize (new java.awt.Dimension (this.MainDisplayPanel.getWidth (), this.MainDisplayPanel.getHeight ()));
            this.plot.setUpPlot ();
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton CloseButton;
    private javax.swing.JPanel ControlsPanel;
    private javax.swing.JPanel MainDisplayPanel;
    private javax.swing.JLabel MouseInfoLabel;
    private javax.swing.JButton SnapshotButton;
    private javax.swing.JLabel jLabel1;
    // End of variables declaration//GEN-END:variables
    
}
