/*
 * FileFilterImage.java
 *
 * Created on June 8, 2007, 9:58 AM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.io.File;
import javax.swing.*;
import javax.swing.filechooser.*;

/* ImageFilter.java is used by FileChooserDemo2.java. */
/**
 * This is an extension of the FileFilter class to filter for save files of the 
 * GeoShear program. The save files for GeoShear are in plain text, but use their
 * own extension. This filter allows both the GeoShear specific files and plain 
 * text files.
 * @author cwarren
 */
public class FileFilterGeoShear extends FileFilter {
    
    /**
     * the string extension used for GeoShear files
     */
    public static String FORMAT_GEOSHEAR_EXTENSION = "ges";    
    /**
     * the string extension used for text files
     */
    public static String FORMAT_TXT_EXTENSION = "txt";    

    
    /**
     * Checks to see whether the give File object passes the filter. It does if the 
     * File is a directory or has a name that ends with an allowed extension.
     * @param f the file to check
     * @return true of the File is a directory or has a name that ends in one of the allowed 
     * extensions, and false otherwise.
     */
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }

        String extension = Utils.getExtension (f);
        if (extension != null) {
            return this.isExtensionOK (extension);
        }

        return false;
    }

    /**
     * Returns a string that describes what this filter does
     * @return a string that describes what this filter does
     */
    public String getDescription() {
        return "GeoShear (*.ges) and text files";
    }

    /**
     * Checks to see if the string passed in is one of the allowed extensions. The 
     * check is case insensitive.
     * @param ext The string to check
     * @return true if the string is one of the allowed extensions, and false otherwise
     */
    public boolean isExtensionOK(String ext)
    {
        ext = ext.toLowerCase ();
        if (ext.equals(FileFilterGeoShear.FORMAT_GEOSHEAR_EXTENSION) ||
            ext.equals(FileFilterGeoShear.FORMAT_TXT_EXTENSION))
        {
            return true;
        }
        return false;
    }
    
}