/*
 * PebbleComparator.java
 *
 * Created on June 6, 2007, 4:51 PM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.util.Comparator;

/**
 *
 * @author cwarren
 */
public class PebbleComparator implements Comparator
{
    
    /** Creates a new instance of PebbleComparator */
    public PebbleComparator ()
    {
    }  
    
    /*
     * Compares its two arguments for order. Returns a negative integer, zero, 
     * or a positive integer as the first argument is less than, equal to, or 
     * greater than the second.
     **/
    public int compare(Object o1, Object o2)
    {
        Pebble p1 = (Pebble)o1;
        Pebble p2 = (Pebble)o2;
                
        // position : first vertical, then horizontal
        if (p1.getCenter ().y < p2.getCenter ().y)
        {
            return -1;
        }
        if (p1.getCenter ().y > p2.getCenter ().y)
        {
            return 1;
        }
        
        if (p1.getCenter ().x < p2.getCenter ().x)
        {
            return -1;
        }
        if (p1.getCenter ().x > p2.getCenter ().x)
        {
            return 1;
        }

        // size : first long axis, then short axis
        if (p1.getLongAxisLength () < p2.getLongAxisLength ())
        {
            return -1;
        }
        if (p1.getLongAxisLength () > p2.getLongAxisLength ())
        {
            return 1;
        }

        if (p1.getShortAxisLength () < p2.getShortAxisLength ())
        {
            return -1;
        }
        if (p1.getShortAxisLength () > p2.getShortAxisLength ())
        {
            return 1;
        }

        // angle : closest to 0
        if (Math.abs (p1.getThetaRad ()) < Math.abs (p2.getThetaRad ()))
        {
            return -1;
        }
        if (Math.abs (p1.getThetaRad ()) > Math.abs (p2.getThetaRad ()))
        {
            return 1;
        }

        return 0;
    }
    
    public boolean equals(Object obj)
    {
        return false;
    }
    
}
