/*
 * PlotPolarLnRfVs2Phi.java
 *
 * Created on June 29, 2007, 10:14 AM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.Graphics2D;

/**
 *
 * @author cwarren
 */
public class PlotPolarLnRfVs2Phi extends PebblePlotterPolar
{
    
    /** Creates a new instance of PlotPolarLnRfVs2Phi */
    public PlotPolarLnRfVs2Phi ()
    {
        super();
    }
    
    protected void plotPebble (Graphics2D g2D, Pebble peb)
    {
        double radialValue = Math.log (peb.getRatioLongToShort ());
        //double radialValue = peb.getRatioLongToShort ();
        double radialDim = (radialValue / this.getRadiusMax ()) * this.getRadiusMaxDim ();
        
        double effectiveAngle = peb.getThetaRad () * 2;
        
        double xOffset = radialDim * Math.cos (effectiveAngle);
        double yOffset = radialDim * Math.sin (effectiveAngle);
        
        int xPos = this.getPlotOrigin ().x + (int)xOffset - this.getMarkSize ()/2;
        int yPos = this.getPlotOrigin ().y + (int)yOffset - this.getMarkSize ()/2;
        
        if (peb.isSelected ())
        {
            Color origColor = g2D.getColor ();
            g2D.setColor (Pebble.SELECTION_PLOT_COLOR);
            
            int markX = xPos-2;
            int markY = yPos-2;
            int markDim = this.getMarkSize ()+4;

            if (this.getMarkShape () == PebblePlotterCartesian.MARK_RECT)
            {
                g2D.fillRect (markX,markY,markDim,markDim);
                //g2D.drawRect (markX,markY,markDim,markDim);
            }
            else if (this.getMarkShape () == PebblePlotterCartesian.MARK_CIRCLE)
            {
                g2D.fillOval (markX,markY,markDim,markDim);
                //g2D.drawOval (markX,markY,markDim,markDim);
            }
            else
            { // defaults to rects
                g2D.fillRect (markX,markY,markDim,markDim);
                //g2D.drawRect (markX,markY,markDim,markDim);
            }
            
            g2D.setColor (origColor);
        }
        
        Color origColor = g2D.getColor ();
        g2D.setColor (peb.getPebbleColor ());
        if (this.getMarkShape () == PebblePlotterCartesian.MARK_RECT)
        {
            if (this.isMarkFill ())
            {
                g2D.fillRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        else if (this.getMarkShape () == PebblePlotterCartesian.MARK_CIRCLE)
        {
            if (this.isMarkFill ())
            {
                g2D.fillOval (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawOval (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        else
        { // defaults to rects
            if (this.isMarkFill ())
            {
                g2D.fillRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        
    }    
}
