/*
 * PlotRfVsPhi.java
 *
 * Created on March 21, 2007, 11:56 AM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;

/**
 *
 * @author cwarren
 */
public class PlotRfVsPhi extends PebblePlotterCartesian
{
    private boolean logScaleX = false;
    
    /** Creates a new instance of PlotRfVsPhi */
    public PlotRfVsPhi ()
    {
        super ();
    }
    
    protected void setUpPlotFrame ()
    {
        super.setUpPlotFrame ();
        Graphics2D g2D = getPlotFrame ().createGraphics ();
        Point po = this.getPlotOrigin ();
        Color origColor = g2D.getColor ();
        g2D.setColor (Color.LIGHT_GRAY);
        g2D.drawLine (po.x+1,po.y-(this.getYAxisSize ()/2),po.x+this.getXAxisSize (),po.y-(this.getYAxisSize ()/2));
        g2D.setColor (origColor);
    }
    
    protected void plotPebble (Graphics2D g2D, Pebble peb)
    {
        int xOffset = 1;
        double effectiveXMax = Utils.truncForDisplay (this.getXMaxValAuto () * 1.2);
        if (this.isManualXMax ())
        {
            effectiveXMax = this.getXMaxValManual ();
        }
        
        if (this.isLogScaleX ())
        {
            //if (this.isManualXMax ())
            //{
                this.setXMaxValLabel (Double.toString (effectiveXMax));
                xOffset = (int)((double)(this.getXAxisSize ()) * (Math.log10 (peb.getRatioLongToShort ()) / (Math.log10 (effectiveXMax)) ));
            //} else
            //{
            //    String labelMax = Double.toString (effectiveXMax); 
            //    this.setXMaxValLabel (labelMax);
            //    xOffset = (int)((double)(this.getXAxisSize ()) * (Math.log10 (peb.getRatioLongToShort ()) / (Math.log10 (effectiveXMax)) ));
            //}
        }
        else
        {
            this.setXMaxValLabel (Double.toString (effectiveXMax));
            xOffset = (int)((double)(this.getXAxisSize ()) * (peb.getRatioLongToShort () / effectiveXMax));
        }
        
        int yOffset = (int)((double)(this.getYAxisSize ()) * ((peb.getThetaRad () * -1.0)/Math.PI)) + this.getYAxisSize ()/2;
        
        int xPos = this.getPlotOrigin ().x + xOffset - (this.getMarkSize ()/2);
        int yPos = this.getPlotOrigin ().y - yOffset - (this.getMarkSize ()/2);

        // show selection highlighting
        if (peb.isSelected ())
        {
            Color origColor = g2D.getColor ();
            g2D.setColor (Pebble.SELECTION_PLOT_COLOR);
            
            int markX = xPos-2;
            int markY = yPos-2;
            int markDim = this.getMarkSize ()+4;

            if (this.getMarkShape () == PebblePlotterCartesian.MARK_RECT)
            {
                g2D.fillRect (markX,markY,markDim,markDim);
                //g2D.drawRect (markX,markY,markDim,markDim);
            }
            else if (this.getMarkShape () == PebblePlotterCartesian.MARK_CIRCLE)
            {
                g2D.fillOval (markX,markY,markDim,markDim);
                //g2D.drawOval (markX,markY,markDim,markDim);
            }
            else
            { // defaults to rects
                g2D.fillRect (markX,markY,markDim,markDim);
                //g2D.drawRect (markX,markY,markDim,markDim);
            }
            
            g2D.setColor (origColor);
        }
        
        Color origColor = g2D.getColor ();
        g2D.setColor (peb.getPebbleColor ());
        if (this.getMarkShape () == PebblePlotterCartesian.MARK_RECT)
        {
            if (this.isMarkFill ())
            {
                g2D.fillRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        else if (this.getMarkShape () == PebblePlotterCartesian.MARK_CIRCLE)
        {
            if (this.isMarkFill ())
            {
                g2D.fillOval (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawOval (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        else
        { // defaults to rects
            if (this.isMarkFill ())
            {
                g2D.fillRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        
        // NOTE: info only shows on selected pebbles
        if (this.isShowRFOnPebble () && peb.isSelected ())
        {
            g2D.setColor (Color.BLACK);
            String rFText = Utils.truncForDisplay (Double.toString (peb.getRatioLongToShort ()));            
/*            rfText = Utils.truncTextDecimal (rfText,3);
            if (rFText.indexOf (".") > 0)
            {
                int truncLength = rFText.length () - rFText.indexOf (".");
                if (truncLength > 4)
                {
                    truncLength = 4;
                }
                rFText = rFText.substring (0,rFText.indexOf (".")+truncLength);
            }
  */          
            String longAxisText = Integer.toString (peb.getLongAxisLength ());            
            String shortAxisText = Integer.toString (peb.getShortAxisLength ());            

            String phiText = Utils.truncForDisplay (Double.toString (peb.getThetaRad ()));
/*            if (phiText.indexOf (".") > 0)
            {
                int truncLength = phiText.length () - phiText.indexOf (".");
                if (truncLength > 4)
                {
                    truncLength = 4;
                }
                phiText = phiText.substring (0,phiText.indexOf (".")+truncLength);
            }
  */          
            //String pebbleInfo = "phi="+phiText+"; rf="+rFText+"=("+longAxisText+"/"+shortAxisText+")";
            String pebbleInfo = "x="+rFText+" y="+phiText;
            g2D.drawString (pebbleInfo,xPos+this.getMarkSize ()+ 3,yPos+this.getMarkSize ());
        }
        g2D.setColor (origColor);
    }
    
    public boolean isLogScaleX ()
    {
        return logScaleX;
    }
    
    public void setLogScaleX (boolean logScaleX)
    {
        this.logScaleX = logScaleX;
        if (logScaleX)
        {
            this.setXLabel ("log10 of "+this.getXLabelOrig ());
        } else
        {
            this.setXLabel (this.getXLabelOrig ());
        }
    }
    
}
